try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

_thread_locals = local()

def get_current_user():
    return getattr(_thread_locals, 'gatekeeper_user', None)

class GatekeeperMiddleware(object):
    def process_request(self, request):
        _thread_locals.gatekeeper_user = getattr(request, 'user', None)


from django.contrib import admin
from gatekeeper.models import ModeratedObject

class ModeratedObjectAdmin(admin.ModelAdmin):
    list_filter = ['status','content_type']

admin.site.register(ModeratedObject, ModeratedObjectAdmin)

if not admin.site.index_template:
    admin.site.index_template = "admin/gatekeeper/index.html"


from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from gatekeeper.models import ModeratedObject
import datetime
import gatekeeper

@staff_member_required
def moderate(request):
    if request.method == 'POST':
        status = 0
        object_ids = request.POST.getlist('objects')
        if "_deny" in request.POST:
            status = -1
        elif "_approve" in request.POST:
            status = 1
        if status and object_ids:
            for obj_id in object_ids:
                if status == 1:
                    ModeratedObject.objects.get(pk=obj_id).approve(request.user)
                elif status == -1:
                    ModeratedObject.objects.get(pk=obj_id).reject(request.user)

    return HttpResponseRedirect(reverse('gatekeeper_moderate_list'))

@staff_member_required
def moderate_list(request, app_label=None, model=None):
    
    content_type = request.GET.get("content_type", None)
    if content_type:
        content_type = ContentType.objects.get(pk=content_type)
    
    pending = ModeratedObject.objects.filter(status=0)
    if content_type:
        pending = pending.filter(content_type=content_type)
        
    cts = [ContentType.objects.get_for_model(model) for model in gatekeeper.registered_models]
        
    data = {
        "content_type": content_type,
        "pending": pending,
        "cts": cts,
    }
    
    return render_to_response('admin/gatekeeper/moderate.html', data)


from django.conf.urls.defaults import *

urlpatterns = patterns('',
    url(r'^moderate/save/$', 'gatekeeper.views.moderate', name="gatekeeper_moderate"),
    url(r'^moderate/(?P<app_label>\w+)\.(?P<model>\w+)/$', 'gatekeeper.views.moderate_list'),
    url(r'^moderate/$', 'gatekeeper.views.moderate_list', name="gatekeeper_moderate_list"),
)


from django.db import models
from django.db.models import signals
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
import datetime
import gatekeeper

STATUS_CHOICES = (
    (1, "Approved"),
    (0, "Pending"),
    (-1, "Rejected"),
)

class ModeratedObjectManager(models.Manager):
    
    def get_for_instance(obj):
        ct = ContentType.objects.get_for_model(obj.__class__)
        try:
            mo = ModeratedObject.objects.get(content_type=ct, object_id=obj.pk)
            return mo
        except ModeratedObject.DoesNotExist:
            pass
            
class ModeratedObject(models.Model):
    
    objects = ModeratedObjectManager()
    
    timestamp = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    
    status = models.IntegerField(choices=STATUS_CHOICES)
    moderated_by = models.ForeignKey(User, blank=True, null=True)
    moderation_date = models.DateTimeField(blank=True, null=True)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    class Meta:
        ordering = ['timestamp']
    
    def __unicode__(self):
        return "[%s] %s" % (self.get_status_display(), self.content_object)
        
    def get_absolute_url(self):
        if hasattr(self.content_object, "get_absolute_url"):
            return self.content_object.get_absolute_url()
        
    def _moderate(self, status, user):
        self.status = status
        self.moderated_by = user
        self.moderation_date = datetime.datetime.now()
        self.save()
        gatekeeper.post_moderation.send(sender=ModeratedObject, instance=self)

    def approve(self, user):
        self._moderate(1, user)

    def reject(self, user):
        self._moderate(-1, user)


from distutils.core import setup

long_description = open('README.rst').read()

setup(
    name='django-gatekeeper',
    version="0.1",
    package_dir={'gatekeeper': 'gatekeeper'},
    packages=['gatekeeper'],
    package_data={'gatekeeper': ['templates/admin/gatekeeper/*.html']},
    description='Django object moderation',
    author='Jeremy Carbaugh',
    author_email='jcarbaugh@sunlightfoundation.com',
    license='BSD License',
    url='http://github.com/sunlightlabs/django-gatekeeper/',
    long_description=long_description,
    platforms=["any"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Environment :: Web Environment',
    ],
)



__author__ = "Jeremy Carbaugh (jcarbaugh@sunlightfoundation.com)"
__version__ = "0.1"
__copyright__ = "Copyright (c) 2008 Sunlight Labs"
__license__ = "BSD"

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.db.models import signals
from django.dispatch import Signal
from gatekeeper.middleware import get_current_user
from gatekeeper.models import ModeratedObject
import datetime

ENABLE_AUTOMODERATION = getattr(settings, "GATEKEEPER_ENABLE_AUTOMODERATION", False)
DEFAULT_STATUS = getattr(settings, "GATEKEEPER_DEFAULT_STATUS", 0)
MODERATOR_LIST = getattr(settings, "GATEKEEPER_MODERATOR_LIST", [])

post_moderation = Signal(providing_args=["instance"])

def _get_automod_user():
    try:
        return User.objects.get(username__exact="gatekeeper_automod")
    except User.DoesNotExist:
        from django.contrib.sites.models import Site
        site = Site.objects.get(id=settings.SITE_ID)
        automod_user = User.objects.create_user(
            'gatekeeper_automod', 'gatekeeper_automod@%s' % site.domain)
        automod_user.save()
        return automod_user

#
# register models with gatekeeper
#
# 

registered_models = {}

def register(model, import_unmoderated=False, auto_moderator=None):
    if not model in registered_models:
        signals.post_save.connect(save_handler, sender=model)
        signals.pre_delete.connect(delete_handler, sender=model)
        registered_models[model] = auto_moderator
        if import_unmoderated:
            try:
                unmod_objs = unmoderated(model.objects.all())
                for obj in unmod_objs:
                    mo = ModeratedObject(
                        status=DEFAULT_STATUS,
                        content_object=obj,
                        timestamp=datetime.datetime.now())
                    mo.save()
            except:
                pass
                
#
# handler for object creation/deletion
#

def save_handler(sender, **kwargs):

    if kwargs.get('created', None):
    
        instance = kwargs['instance']
    
        mo = ModeratedObject(
            status=DEFAULT_STATUS,
            content_object=instance,
            timestamp=datetime.datetime.now())
        mo.save()
    
        if ENABLE_AUTOMODERATION:
        
            auto_moderator = registered_models[instance.__class__]
            if auto_moderator:
                mod = auto_moderator(mo)
                if mod is None:
                    pass # ignore the moderator if it returns None
                elif mod:
                    mo.approve(_get_automod_user())
                else:
                    mo.reject(_get_automod_user())
        
            if mo.status == 0: # if status is still pending
                user = get_current_user()
                if user and user.is_authenticated():
                    if user.is_superuser or user.has_perm('gatekeeper.change_moderatedobject'):
                        mo.approve(user)
                    
        if MODERATOR_LIST:
            subject = "[pending-moderation] %s" % instance
            message = "New object pending moderation.\n%s\n%s" % (instance, reverse("gatekeeper_moderate_list"))
            from_addr = "bounce@sunlightfoundation.com"
            send_mail(subject, message, from_addr, MODERATOR_LIST, fail_silently=True)

def delete_handler(sender, **kwargs):
    instance = kwargs['instance']
    try:
        ct = ContentType.objects.get_for_model(sender)
        mo = ModeratedObject.objects.get(content_type=ct, object_id=instance.pk)
        mo.delete()
    except ModeratedObject.DoesNotExist:
        pass

#
# filter querysets
#

def approved(qs):
    return _by_status(1, qs)

def pending(qs):
    return _by_status(0, qs)

def rejected(qs):
    return _by_status(-1, qs)

def unmoderated(qs):
    return _by_status(None, qs)

def _by_status(status, qs):
    if hasattr(qs, "model"):
        ct = ContentType.objects.get_for_model(qs.model)
        if status:
            ids = ModeratedObject.objects.filter(content_type=ct, status=status).values_list('object_id', flat=True)
            qs = qs.filter(pk__in=ids)
        else:
            ids = ModeratedObject.objects.filter(content_type=ct).values_list('object_id', flat=True)
            qs = qs.exclude(pk__in=ids)
    return qs